//log.cpp
//log [network] physics packets in order to replay the games

#include <stdlib.h>
#include "networkphysics.h"
#include "gamemem.h"
#include "log.h"
#include "gameinitexit.h"
#include "gameframe.h"
#include "network.h"

typedef struct{
	int offset;
	short size;
	short type;
} tLogEntry;

#define kLogSize (5*1024*1024)
#define kGhostLogSize (1024*1024)
#define kIndexSize (1024*100)

char gPacketLog[kLogSize];
char gGhostLog[kGhostLogSize];
tLogEntry gPacketLogIndex[kIndexSize];
tLogEntry gGhostIndex[kIndexSize];
int gLogPos=0;
int gIndexPos=0;
int gGhostLogPos=0;
int gGhostIndexPos=0;

void LogPacket(int size, void *data,int type)
{
	if(gIndexPos<kIndexSize)
		if(gLogPos+size<kLogSize)
		{
			gPacketLogIndex[gIndexPos].size=size;
			gPacketLogIndex[gIndexPos].offset=gLogPos;
			gPacketLogIndex[gIndexPos].type=type;
			gIndexPos++;
			MemoryMove(gPacketLog+gLogPos,data,size);
			gLogPos+=size+sizeof(int);
		}
}

void LogToGhostLog()
{
//kLogSize>kGhostLogSize!!!!
	MemoryMove(gGhostLog,gPacketLog,gLogPos>kGhostLogSize?kGhostLogSize:gLogPos);
	MemoryMove(gGhostIndex,gPacketLogIndex,gIndexPos*sizeof(tLogEntry));
	gGhostLogPos=gLogPos;
	gGhostIndexPos=gIndexPos;
}

int LogCompare(const void *a,const void *b)
{
	return ((tPhysicsMessage*)a)->frame-((tPhysicsMessage*)b)->frame;
}

void LogSort()
{
	qsort(gPacketLogIndex,gIndexPos,sizeof(tLogEntry),LogCompare);
}

void LogReset()
{
	gLogPos=0;
	gIndexPos=0;
}

void GhostLogReset()
{
	gGhostLogPos=0;
	gGhostIndexPos=0;
}


void *LogGetPacket(int index,int *type,int log)
{
	if(log==kLogReplayLog||gGhostLogPos==0)
		if(index<gIndexPos)
		{
			*type=gPacketLogIndex[index].type;
			return gPacketLog+gPacketLogIndex[index].offset;
		}
		else 
			return 0;
	else
		if(index<gGhostIndexPos)
			if(gGhostIndex[index].offset<kGhostLogSize)
			{
				*type=gGhostIndex[index].type;
				return gGhostLog+gGhostIndex[index].offset;
			}
	return 0;
}

void GameInfoSend(tGameInfo *gInfo);
void GameInfoReceive(tGameInfo *gInfo);
void ConvertGInfoSwap(tGameInfo *gInfo);

void LogSaveToDisk()
{
	FILE* f=fopen("replay.redlog","w");
	SysBeep(0);
	GameInfoSend(gGameInfo);
	fwrite(gGameInfo,sizeof(tGameInfo),1,f);
	if(gGameInfo->numLaps==-1)
	{
		fwrite(&gGhostIndexPos,sizeof(int),1,f);
		fwrite(&gGhostLogPos,sizeof(int),1,f);
		fwrite(&gReplayOldFrameCount,sizeof(int),1,f);
		fwrite(gGhostIndex,sizeof(tLogEntry),gGhostIndexPos,f);
		for(int i=0;i<gGhostIndexPos;i++)
		{
			switch(gGhostIndex[i].type)
			{
				case kMessageTypePhysics:
					{
						tPhysicsMessage *physMessage=(tPhysicsMessage*)(gGhostLog+gGhostIndex[i].offset);
						physMessage->frame-=(gBestLapTime!=0?gBestLapStart+5:gCurrentLapStart+5);
					}
			}
		}
		fwrite(gGhostLog,sizeof(char),gGhostLogPos,f);
		for(int i=0;i<gGhostIndexPos;i++)
		{
			switch(gGhostIndex[i].type)
			{
				case kMessageTypePhysics:
					{
						tPhysicsMessage *physMessage=(tPhysicsMessage*)(gGhostLog+gGhostIndex[i].offset);
						physMessage->frame+=(gBestLapTime!=0?gBestLapStart+5:gCurrentLapStart+5);
					}
			}
		}
	}
	else
	{
		fwrite(&gIndexPos,sizeof(int),1,f);
		fwrite(&gLogPos,sizeof(int),1,f);
		fwrite(&gReplayOldFrameCount,sizeof(int),1,f);
		fwrite(gPacketLogIndex,sizeof(tLogEntry),gIndexPos,f);
		fwrite(gPacketLog,sizeof(char),gLogPos,f);
	}
	fclose(f);
}

#define S32ChangeToNative(value) ((value)=EndianS32_BtoN(value))
#define U32ChangeToNative(value) ((value)=EndianU32_BtoN(value))
#define S32ChangeToNet(value) ((value)=EndianS32_NtoB(value))
#define U32ChangeToNet(value) ((value)=EndianU32_NtoB(value))
#define S16ChangeToNative(value) ((value)=EndianS16_BtoN(value))
#define U16ChangeToNative(value) ((value)=EndianU16_BtoN(value))
#define S16ChangeToNet(value) ((value)=EndianS16_NtoB(value))
#define U16ChangeToNet(value) ((value)=EndianU16_NtoB(value))


void LogLoad(tFileRef f,tGameInfo* gInfo)
{
	char *ch=(char*)FileGetDataPtr(f);
	
	*gInfo=*(tGameInfo*)ch;
	ConvertGInfoSwap(gInfo);
	GameInfoReceive(gInfo);
	ch+=sizeof(tGameInfo);//-sizeof(UInt32)*kMaxPlayers;
	
	gIndexPos=*(int*)ch;
	S32ChangeToNative(gIndexPos);
	gGhostIndexPos=gIndexPos;
	ch+=sizeof(int);

	gLogPos=*(int*)ch;
	S32ChangeToNative(gLogPos);
	ch+=sizeof(int);

	gReplayOldFrameCount=*(int*)ch;
	S32ChangeToNative(gReplayOldFrameCount);
	ch+=sizeof(int);

	MemoryMove(gPacketLogIndex,ch,sizeof(tLogEntry)*gIndexPos);
	for(int i=0;i<gIndexPos;i++)
	{	
		S32ChangeToNative(gPacketLogIndex[i].offset);
		S16ChangeToNative(gPacketLogIndex[i].type);
		S16ChangeToNative(gPacketLogIndex[i].size);
	}
	ch+=sizeof(tLogEntry)*gIndexPos;

	MemoryMove(gPacketLog,ch,sizeof(char)*gLogPos);
}
